<?php

namespace App\Facade\Filters;

use Error;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Exceptions\HttpResponseException;
use JsonMapper;

class Filters
{
    /**
     * @var Filter[]
     */
    public array $filters;

    /**
     * @var Order[]
     */
    public array $orders;

    public ?Pagination $pagination;

    public static function fromArrayOrObject(array|object $data): Filters
    {
        $mapper = new JsonMapper;
        if (is_array($data)) {
            $data = json_decode(json_encode($data), false);
        }

        $data->filters ??= [];
        $data->orders ??= [];
        $data->pagination ??= null;

        try {
            return $mapper->map($data, \App\Facade\Filters\Filters::class);
        } catch (Error $err) {
            throw new HttpResponseException(response()->json(['unknown' => $err->getMessage()], 422));
        }
    }

    public function apply(Builder $builder)
    {
        foreach ($this->filters as $filter) {
            switch ($filter->type) {
                case FilterTypeEnum::Is:
                    $builder->where($filter->column, $filter->filter);
                    break;
                case FilterTypeEnum::Not:
                    $builder->whereNot($filter->column, $filter->filter);
                    break;
                case FilterTypeEnum::Like:
                    $builder->whereLike($filter->column, $filter->filter);
                    break;
            }
        }

        foreach ($this->orders as $order) {
            $builder->orderBy($order->by, $order->sort->value);
        }

        if (! is_null($this->pagination)) {
            $builder->paginate($this->pagination->size, ['*'], 'page', $this->pagination->page);
        }
    }
}
